from xml.dom import minidom
from xml.etree import ElementTree as Etree
import hashlib


class Response:
    def __init__(self, xml=None):
        """
        Defines a Response object
        :param xml: a string containing xml data
        """
        if xml:
            self._xml = Etree.fromstring(xml)
        else:
            self._xml = None

    def __getattr__(self, item):
        """
        Search unidentified attributes inside the ElementTree structure inside the _xml attribute.
        :param item: the name of the requested attribute
        :return: the content of the first tag named item in the ElementTree structure
        """
        try:
            return self._xml.find(item).text
        except AttributeError:
            raise AttributeError(item+' not found in response.')

    @property
    def timestamp(self):
        """
        Gets the timestamp of the response, located in an attribute of the root element.
        :return: a string representing a timestamp.
        """
        return self._xml.attrib.get('timestamp', '')

    def list_xml_tags(self):
        """
        Returns a list of all xml tags inside the object.
        :return: list of strings representing the tags of the xml elements contained by the object.
        """
        return [elem.tag for elem in self._xml.iter()]

    def to_xml_string(self):
        """
        Turns the xml tree contained by this object into a string.
        :return: a string with the xml contained by this object.
        """
        binary = Etree.tostring(self._xml, encoding='utf8', method='xml')
        return binary.decode('utf-8')

    def to_pretty_xml(self):
        """
        Turns the xml tree contained by this object into a pretty string for easy reading.
        :return: a string with the xml contained by this object.
        """
        return minidom.parseString(self.to_xml_string()).toprettyxml()

    def _hash(self):
        """
        Builds the response hash from the data contained within
        :return: the hash string that will latter be cyphered
        """
        try:
           pasref = self.pasref or ''
        except AttributeError:
           pasref = ''
        try:
           authcode = self.authcode or ''
        except AttributeError:
           authcode = ''
        res = "%s.%s.%s.%s.%s.%s.%s" % (str(self.timestamp), str(self.merchantid), str(self.orderid), str(self.result)
                                       , str(self.message),str(pasref),str(authcode))
        return res.encode('utf-8')

    def _sha1_hash(self, secret):
        """
        returns a secure hash in SHA-1 for this response
        :param secret: the shared secret between Elavon and your account
        :return: secure hash in SHA-1
        """
        sha1_hash = hashlib.sha1(self._hash()).hexdigest()
        sha1_hash += ".%s" % secret

        return hashlib.sha1(sha1_hash.encode('utf-8')).hexdigest()

    def validate_origin(self, secret):
        """
        validates the request hash to check if the response was really sent by Elavon
        :param secret: the shared secret between Elavon and your account
        :return: a boolean value indicating the validity of the response hash
        """
        try:
            return self.sha1hash == self._sha1_hash(secret)
        except AttributeError:
            return False



from elavonvtpv.enum import CVNIndicator
from xml.etree import ElementTree as ETree


class CreditCard:
    def __init__(self, cvn_indicator, number, expiry, name, card_type, cvn=None):
        """
        Defines a CreditCard object
        :param cvn_indicator: CVNIndicator enum object indicating the presence or lack of thereof of the cvn
        :param number: the credit card number
        :param expiry: the credit card expiration date
        :param name: the name of the credit card titular
        :param card_type: CardType enum object indicating the type of the credit card
        :param cvn: the credit card validation code
        """
        self.cvn_indicator = cvn_indicator
        self.number = number
        self.expiry = expiry
        self.name = name
        self.card_type = card_type
        self.cvn = cvn

    def to_etree_element(self):
        """
        Transforms the CreditCard object into an ElementTree Element so it can be used in the creation of a XML
        :return: An ElmentTree Element containing the data on the CreditCard object
        """
        card = ETree.Element('card')

        presind = ETree.SubElement(card, 'presind')
        presind.text = self.cvn_indicator.value

        number = ETree.SubElement(card, 'number')
        number.text = self.number

        expdate = ETree.SubElement(card, 'expdate')
        expdate.text = self.expiry

        chname = ETree.SubElement(card, 'chname')
        chname.text = self.name

        card_type = ETree.SubElement(card, 'type')
        card_type.text = self.card_type.value

        if self.cvn_indicator == CVNIndicator.present:
            cvn = ETree.SubElement(card, 'cvn')
            cvn__number = ETree.SubElement(cvn, 'number')
            cvn__number.text = self.cvn

        return card



from enum import Enum


class RequestType(Enum):
    auth = 1
    manual = 2
    obt = 3
    offline = 4
    rebate = 5
    void = 6
    TSS = 7
    settle = 8
    verify_enrolled = 9
    verify_sig = 10


class Currency(Enum):
    euro = 'EUR'
    us_dollar = 'USD'
    br_real = 'BRL'
    mx_peso = 'MXN'


class Channel(Enum):
    e_commerce = 'ECOM'
    telephone = 'MOTO'
    email = 'MOTO'


class CardType(Enum):
    mastercard = 'MC'
    visa = 'VISA'
    american_express = 'AMEX'


class CVNIndicator(Enum):
    present = '1'
    illegible = '2'
    not_present = '3'
    not_requested = '4'



from elavonvtpv.enum import CVNIndicator
from xml.etree import ElementTree as ETree


class Mpi:
    def __init__(self, eci, cavv=None, xid=None):
        """
        Defines a Mpi object
        :param cavv: the CAVV or UCAF value necessary for the authorization request
        :param xid: the XID field necessary for the authorization request
        :param eci: the electronic commerce indicator necessary for the authorization request
        """
        self.cavv = cavv
        self.xid = xid
        self.eci = eci

    def to_etree_element(self):
        """
        Transforms the Mpi object into an ElementTree Element so it can be used in the creation of a XML
        :return: An ElmentTree Element containing the data on the Mpi object
        """
        mpi = ETree.Element('mpi')

        if self.cavv:
            cavv = ETree.SubElement(mpi, 'cavv')
            cavv.text = self.cavv

        if self.xid:
            xid = ETree.SubElement(mpi, 'xid')
            xid.text = self.xid

        eci = ETree.SubElement(mpi, 'eci')
        eci.text = self.eci

        return mpi



from elavonvtpv.CreditCard import CreditCard
from elavonvtpv.Mpi import Mpi
from elavonvtpv.enum import RequestType, CVNIndicator, CardType, Currency, Channel
from elavonvtpv.TssInfo import TssInfo
from elavonvtpv.Request import Request
from elavonvtpv.Response import Response
